open Printf
type status = PASSED | FAILED of string option
type test = TEST of (unit -> status) * string

let validate (chan, theTest) = match theTest with
    TEST(theFunction,theName) ->
      begin
	fprintf chan "Running %s..." theName; flush chan;
	(let status = match theFunction () with
	     PASSED -> fprintf chan "PASSED\n"; true
	   | FAILED(None) -> fprintf chan "FAILED\n"; false
	   | FAILED(Some(msg)) -> fprintf chan "FAILED: %s\n" msg; false
	 in flush chan; status);
      end

(* run all validation tests and return true if they all pass *)
let validateAll chan tests = List.fold_left
			       (fun t1 t2 -> validate(chan, t2) && t1)
			       true
			       tests
(* return a status for the comparison of expected and actual,
   falling within a margin of error *)
let expectFloat (expected, actual, margin, msg) =
  let diff = expected -. actual in
  let absdiff = sqrt (diff *. diff) in
   if (absdiff <= margin) then PASSED
   else FAILED(Some(sprintf "%s: EXPECTED %.1f GOT %.1f" msg expected actual))

let expectInt (expected, actual, margin, msg) =
  let diff = expected - actual in
  let absdiff = diff * diff in
   if (absdiff <= (margin * margin)) then PASSED
   else FAILED(Some(sprintf "%s: EXPECTED %d GOT %d" msg expected actual))

let expectString (expected, actual,msg) =
  match (expected = actual) with
      true -> PASSED
    | false -> FAILED(Some(sprintf "%s:\n%s\nNOT THE SAME AS\n%s"
			     msg expected actual))

(* run all of the tests and return a UNIX completion code *)
let doTests outchan tests =
  try
    match (validateAll outchan tests) with
	true -> fprintf outchan "PASSED all tests\n"; flush outchan; 0
      | false -> fprintf outchan "FAILED some tests\n"; flush outchan; 1
  with Sys_error msg -> fprintf outchan "%s\n" msg; flush outchan; 1

