module StringMap: Map.S with type key = string

type expr =
  | OID of string
  | IDENT of string
  | INT of int
  | FLOAT of float
  | FACT of string * expr list
  | ASSOC of string * string * string

type oid_tbl = expr StringMap.t
type fact_tbl = expr StringMap.t
type tbl_map = fact_tbl StringMap.t
type facts_data = oid_tbl * tbl_map * (string StringMap.t)

val print: expr list -> unit
val string_of_expr: expr -> string

val organize_facts: expr list -> facts_data
val tbl_names: facts_data -> string list
val get_rows: string -> tbl_map -> expr list

(* lookup a fact by its OID string *)
val get_fact: facts_data -> string -> expr

(* get the primary key name of a fact name *)
val primaryKeyName: facts_data -> string -> string
val primaryKey: facts_data -> expr -> string

(* get the string value of the field for the specified oid
 * field_value fact-tables key oid
 *)
val field_value: facts_data -> string -> string -> string
