{
  open Facts_parser  (* token definitions *)
  
  (* little helpers *)
  let get   = Lexing.lexeme

  (* simple minded definitions for line counting. Whenever we
   * we scan a newline we call newline().
   *)

  let line         = ref 1
  let startOfLine  = ref 0
  let prefix str n = String.sub str 0 (String.length str - n)

  let init () =
    line          := 1;
    startOfLine   := 0

  let newline lexbuf =
    line := !line + 1;
    startOfLine := Lexing.lexeme_start lexbuf

  (* actual (line, column) position *)
  let position lexbuf = (!line,Lexing.lexeme_start lexbuf - !startOfLine)
}

(* regexp declarations *)

let     ws     = [' ' '\t' '\r']
let     nl     = '\n'
let     ident  = ['a'-'z' 'A'-'Z'] + ['a'-'z' 'A'-'Z' '0'-'9' '-' '_'] *
let     digit  = ['0'-'9']
let     h      = ['0'-'9' 'a'-'f' 'A'-'F']
let     oid    = h h h h '-' h h h h '-' h h h h '-' h h h h
let     assoc  = "primaryKeyAssociation" | "assoc"

rule token = parse
      eof			{ EOF }
   
   (* whitespace and newlines *)
   | nl	  		        { newline lexbuf; token lexbuf }
   | ws	+                       { token lexbuf }

   | assoc                      { ASSOC(get lexbuf) }
   | '"'			{ string1 lexbuf }
   | oid			{ OID(get lexbuf) }
   | ident			{ IDENT(get lexbuf) }
   | digit+                     { INT(int_of_string (get lexbuf)) }
   | digit+ '.' digit+          { FLOAT(float_of_string (get lexbuf)) }
   | '('			{ LP }
   | ')'			{ RP }
   | '-'			{ MINUS }

   (* everything else is illegal *)
   | _				{ failwith ("illegal character") }

and string1 = parse
   | [^ '"' '\n']* '"'          { IDENT(prefix (get lexbuf) 1)}
   | [^ '"' '\n']* eof		{ failwith "unterminated string"   }
   | [^ '"']* nl	        { failwith "newline in string is unsupported"}
   | _				{ failwith "illegal char in string"}

