type value =
  | LIST of value list
  | STRING of string
  | BOOL of bool
  | INT of int
  | ASSOC of env
and env = (string * value) list

type id = string
type reference =
  | REF of id
  | DOTREF of id * reference

type text = string

type range_expr =
  | REF_RANGE of reference
  | INT_RANGE of int

type prim_expr =
  | REF_EXPR of reference
  | STR_EXPR of string
  | INT_EXPR of int
  | BOOL_EXPR of bool
  | LIST_EXPR of prim_expr list
  | RANGE_EXPR of range_expr * range_expr

type expr =
  | PRIM of prim_expr
  | EQ of expr * expr
  | NE of expr * expr
  | GT of expr * expr
  | GE of expr * expr
  | LT of expr * expr
  | LE of expr * expr
  | AND of expr * expr
  | OR of expr * expr
  | ADD of expr * expr
  | SUB of expr * expr
  | MUL of expr * expr
  | DIV of expr * expr
  | MOD of expr * expr
  | NOT of expr

type stmt =
  | SCHMOO of text
  | BLOCK of stmt list
  | IF_STMT of expr * stmt * stmt option
  | FOR_STMT of id * prim_expr * stmt
  | SET_STMT of reference * expr
  | REF_STMT of reference
  | INC_STMT of prim_expr list (* list and int types are bogus here *)
  | PARSE_STMT of prim_expr (* list and string types are bogus here *)

type template = stmt list

val empty: env
val lookup: env -> reference -> value
val extend: env -> reference -> value -> env
val extendlist: env -> (reference * value) list -> env
val augment: env -> env -> env
val remove: env -> reference -> env
val name_of_var: reference -> string
val string_of_var: env -> reference -> string
val string_of_pexpr: expr:prim_expr -> string
val string_of_expr: expr:expr -> string
val print_stmt: stmt -> unit
val string_of_env: env -> string
