type uri = string
type name = string
type value = string
type attr = name * value
type params = attr list

(* service functions take a [path] and [query params] and return
 * a VTL environment. The engine will be called with the matching
 * VTL template and the output generated.
 *)
type srvc_func = (string * params) -> Vtl.env

(* template name * service function *)
type service = string * srvc_func

(* services association defines the whole application *)
type services = (name * service) list


(* split a URL into its path and the environment of parameters *)
val splitRequest: uri -> string * params

(* build a query URI from a list of attributes *)
val queryUri: string * attr list -> string

(* return a value from the parameter environment *)
val getEnv: params -> string -> string

(* return all values from the params as a list *)
val getEnvAll: env:params -> name:string -> string list

(********************************************************************** 
 * server (port:int) (template_root:string) (services:services)
 *    (not_found_srvc:service)
 *
 * An HTTP server will be started on the supplied [port]. URI requests
 * are split into path and query parameters and the [services] assoc
 * searched for a matching pathname-service. If it is found, it will
 * be called with the arguments (path * params) and expected to
 * produce a Vtl.env properties binding. The supplied [template] will
 * be searched under [template_root] and evaluated to produce a string
 * document which is printed to stdout.
 *
 * If the path has no association in the services list, the not-found
 * service function is called with the original URI and params.
 **********************************************************************)
val server: int -> string -> services -> service -> unit

type error_handler = string*params -> exn -> unit
val customServer: int -> string -> Vtl.env -> services -> service
  -> error_handler-> unit

val replace: char -> string -> string -> string
